package com.franckbarbier.vegetables;

public class Main {

    public static void main(String[] args) {
        Garden<?> g0 = new Garden<>(); // Any kind of 'Vegetable'...
        Garden<Vegetable.Salad> g1 = new Garden<>();
        Garden<Vegetable.Batavia> g2 = new Garden<>();
        Garden<Vegetable.Lettuce> g3 = new Garden<>();

        // Compilation error because 'Garden&lt;Vegetable.Salad&gt;' is not "similar" to 'Garden&lt;Vegetable.Batavia&gt;':
        // g1 = g2;
        g0 = g2; // Prior 'g0' moves to garbage collection...

        // Java does not support this notation because of "erasure":
        // assert (Garden<Vegetable.Salad>.class != Garden<Vegetable.Batavia>.class);
        // Compilation error because 'Garden<Vegetable.Salad>' and 'Garden<Vegetable.Batavia>' *ARE NOT* related to each other:
        // System.out.println(g1.is_bigger_than_V1(g2)); 
        System.out.println(g1.is_bigger_than_V2(g2)); // Great, no compilation error
        System.out.println(g2.is_bigger_than_V2(g1)); // Great, no compilation error

        // Sound compilation error because 'g2' may get salads, which *ARE NOT* batavias:
        // g1.transfer_V1(g2);
        // Compilation error (unexpected?):
        // g2.transfer_V1(g1); // 'g1' only accepts *DIRECT* salads and not subtypes: Batavia, Lettuce...
        g2.transfer_V2(g1); // Solution: 'g1' gets batavias from 'g2'!

        assert (g1.getClass() == g2.getClass());
        assert (g1 instanceof Garden);
        assert (g2 instanceof Garden);
        assert (g1 instanceof Garden<?>);
        assert (g2 instanceof Garden<?>);
    }
}
